/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

/*
 * Created on Jan 26, 2005
 */
package com.sunlabs.archsim.gasp.terminals;



import com.sunlabs.archsim.base.hierarchy.CompositeEntity;
import com.sunlabs.archsim.base.simulation.Command;
import com.sunlabs.archsim.gasp.ArbitrationParticipant;
import com.sunlabs.archsim.gasp.Arbiter;


/**
 * @author ib27688
 *
 * Representation of a terminal where arbitration is attached to
 * a set of terminals that are declared as conneced.
 * Note that each terminal can belong to only one set of
 * "mutually exclusive peers".
 */
public class ArbiterInput extends InputTerminal 
						implements ArbitrationParticipant {
	
	
	/** Is there an outstanding request
	 */
	protected boolean requestOut;

	/** Did this action receive a grant from the arbiter? */
	protected boolean requestGranted;
	
	/** Arbiter */
	protected Arbiter arbiter;
	
	
	public ArbiterInput(String name) {
		super(name);
		requestOut= false;
		requestGranted= false;
		arbiter= new Arbiter(this, new RequestGrantedCommand());
	} // ArbiterInput

	public ArbiterInput(String name, CompositeEntity g) {
		this(name);
		setParent(g);
	} // ArbiterInput
	
	
	/** Attach an peer arbiter input.
	 * @param ai ArbiterInput to be attached
	 */
	public boolean attachPeer(ArbiterInput ai) {
		return arbiter.merge(ai);
	} // attachPeer
		
	/** Schedule a notification for an action that the state is active.
	 *  First need to obtain a grant from the arbiter.
	 */
	public void notifyStateIsActive() {
		if (!requestOut && !requestGranted) {
			arbiter.request(this);
		}
		else {
			// a request has been issued and a new request arrived
			// without the original request either being granted
			director.fatalError("Two arbitration requests in a row by "
					+ getAlias() + ", id= " + getID());
		}
	} // notifyStateIsActive

	/** Schedule a notification for an action that a state is inactive */
	public void notifyStateIsInactive() {
		// if a request is out, it has to be withdrawn
		if (requestOut) {
			arbiter.withdraw(this);
		}
		else if (requestGranted) {
			// this is bad timimg as the request is being propagated
			// towards the action
			director.fatalError("State" + attachedState.getAlias()  
					+ ", id= " + attachedState.getID()
					+ " became inactive while terminal " + getAlias()
					+ ", id= " + getID() + ", is notifying action "
					+ attachedAction.getAlias() + ", id= " 
					+ attachedAction.getID() 
					+ ", that the state was active");
		}
		else {
			// schedule the notification that the state is inactive
			notifyStateIsInactive();
		}
	} // notifyStateIsInactive
	

	
	/** Schedule setting the state inactive.
	 *
	 */
	public void setStateInactive() {
		if (requestOut) {
			
		}
		if (requestGranted) {
			// about to release the grant
			requestGranted= false;
			// release the arbiter
			arbiter.release(this);
			// inform the state that it is inactive
			setStateInactive();
		}
		else {
			// problem - how did the attached action fire if the
			// lock was not set?
			director.fatalError(getAlias() + ", id= " + getID()
					+ ": attempting to set state " 
					+ attachedState.getAlias() + ", id "
					+ attachedState.getID()
					+ ", inactive without holding an arbiter grant");
		}
	} // setStateInactive
	
	protected class RequestGrantedCommand extends Command {

		public void execute(Object params) {
			requestGranted= true;
			requestOut= false;
			
			// request is siiued only when the action needs
			// to be notified that the state is active
			// therefore, schedule the notification
			notifyStateIsActive();
			
		} // execute
		
	} // RequestGrantedCommand

	
	public Arbiter getArbiter() {
		return arbiter;
	}

	public void setArbiter(Arbiter a) {
		arbiter= a;
	}
	
} // class ArbiterInput
